export declare enum BlockType {
    comment = 0,
    single = 1,
    iface = 2
}
export interface BlockBase {
    type: BlockType;
    _orig: BlockBase | null;
    text: string;
}
export interface CommentBlock extends BlockBase {
    type: BlockType.comment;
    _orig: CommentBlock | null;
}
export interface SingleLineBlock extends BlockBase {
    type: BlockType.single;
    _orig: SingleLineBlock | null;
    key: string;
    options: string[];
}
export interface Setting {
    comment: boolean;
    key: string;
    value: string;
}
export interface IfaceBlock extends BlockBase {
    type: BlockType.iface;
    _orig: IfaceBlock | null;
    iface: string;
    options: string[];
    settings: Setting[];
}
export declare type Block = CommentBlock | SingleLineBlock | IfaceBlock;
export declare class IfUpdownInterfaces {
    private _filename;
    private _blocks;
    private _modifiedBlocks;
    open(filename: string): Promise<void>;
    readFromText(content: string): void;
    saveToText(): string;
    save(filename?: string | undefined): Promise<void>;
    get blocks(): Block[];
    findInterface(iface: string): IfaceBlock | null;
    addSingleLine(key: string, options: string[]): void;
    addInterface(iface: string, options: string[], settings: Setting[]): void;
    removeInterface(iface: string): boolean;
}
export default IfUpdownInterfaces;
//# sourceMappingURL=index.d.ts.map