"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.IfUpdownInterfaces = exports.BlockType = void 0;
var fs = __importStar(require("fs"));
var util = __importStar(require("util"));
var lodash_1 = __importDefault(require("lodash"));
var BlockType;
(function (BlockType) {
    BlockType[BlockType["comment"] = 0] = "comment";
    BlockType[BlockType["single"] = 1] = "single";
    BlockType[BlockType["iface"] = 2] = "iface";
})(BlockType = exports.BlockType || (exports.BlockType = {}));
function blockIsChanged(block) {
    var keys = Object.keys(block)
        .filter(function (v) { return v !== '_orig'; });
    var orig = block._orig;
    if (!orig)
        return true;
    return !keys.reduce(function (prev, key) {
        return prev && lodash_1.default.isEqual(block[key], orig[key]);
    }, true);
}
var IfUpdownInterfaces = /** @class */ (function () {
    function IfUpdownInterfaces() {
        this._filename = '';
        this._blocks = [];
        this._modifiedBlocks = [];
    }
    IfUpdownInterfaces.prototype.open = function (filename) {
        var _this = this;
        this._filename = filename;
        return util.promisify(fs.readFile)(filename, { encoding: 'utf8' })
            .then(function (content) {
            try {
                _this.readFromText(content);
            }
            catch (e) {
                return Promise.reject(e);
            }
        });
    };
    IfUpdownInterfaces.prototype.readFromText = function (content) {
        var lines = content.split('\n')
            .map(function (v) { return v.trimRight(); });
        var currentIface = null;
        this._blocks = [];
        for (var i = 0; i < lines.length; i++) {
            var origLine = lines[i];
            var line = origLine.trim();
            // COMMENT
            if (/^#/.test(line) || line.length <= 0) {
                if (currentIface) {
                    currentIface.settings.push({
                        comment: true,
                        key: '',
                        value: origLine
                    });
                    currentIface.text += "\n".concat(origLine);
                }
                else {
                    var block = {
                        type: BlockType.comment,
                        _orig: null,
                        text: origLine
                    };
                    this._blocks.push(block);
                }
                continue;
            }
            // Single line configuration (not iface)
            var m = /^(auto|source|allow-hotplug)\s+(.+)$/.exec(line);
            if (m) {
                var block = {
                    type: BlockType.single,
                    _orig: null,
                    text: origLine,
                    key: m[1],
                    options: m[2].split(' ').map(function (v) { return v.trim(); })
                };
                this._blocks.push(block);
                currentIface = null;
                continue;
            }
            // iface
            m = /^iface\s+([^ ]+)\s+(.+)$/.exec(line);
            if (m) {
                var iface = m[1];
                var block = {
                    type: BlockType.iface,
                    _orig: null,
                    text: origLine,
                    iface: iface,
                    options: m[2].split(' ').map(function (v) { return v.trim(); }),
                    settings: []
                };
                this._blocks.push(block);
                currentIface = block;
                continue;
            }
            if (currentIface) {
                m = /^([^ ]+)\s+(.+)$/.exec(line);
                if (m) {
                    currentIface.settings.push({
                        comment: false,
                        key: m[1],
                        value: m[2]
                    });
                    currentIface.text += "\n".concat(origLine);
                }
                continue;
            }
            throw new Error('PARSE FAILED: ' + line);
        }
        this._blocks.forEach(function (block) {
            block._orig = lodash_1.default.cloneDeep(block);
        });
        this._modifiedBlocks = lodash_1.default.cloneDeep(this._blocks);
    };
    IfUpdownInterfaces.prototype.saveToText = function () {
        return this._modifiedBlocks.map(function (block) {
            if (blockIsChanged(block)) {
                if (block.type === BlockType.single) {
                    return block.key + ' ' + block.options.join(' ');
                }
                else if (block.type === BlockType.iface) {
                    return "iface ".concat(block.iface, " ").concat(block.options.join(' '), "\n") +
                        block.settings
                            .map(function (setting) {
                            if (setting.comment) {
                                return "    ".concat(setting.value);
                            }
                            else {
                                return "    ".concat(setting.key, " ").concat(setting.value);
                            }
                        })
                            .join('\n');
                }
                else {
                    return block.text;
                }
            }
            else {
                return block.text;
            }
        })
            .join('\n') + '\n';
    };
    IfUpdownInterfaces.prototype.save = function (filename) {
        var data = this.saveToText();
        return util.promisify(fs.writeFile)(filename || this._filename, data);
    };
    Object.defineProperty(IfUpdownInterfaces.prototype, "blocks", {
        get: function () {
            return this._modifiedBlocks;
        },
        enumerable: false,
        configurable: true
    });
    IfUpdownInterfaces.prototype.findInterface = function (iface) {
        return this._modifiedBlocks.find(function (v) { return v.type === BlockType.iface && v.iface === iface; });
    };
    IfUpdownInterfaces.prototype.addSingleLine = function (key, options) {
        this._modifiedBlocks.push({
            type: BlockType.single,
            _orig: null,
            key: key,
            options: options,
            text: ''
        });
    };
    IfUpdownInterfaces.prototype.addInterface = function (iface, options, settings) {
        this._modifiedBlocks.push({
            type: BlockType.iface,
            _orig: null,
            iface: iface,
            options: options,
            settings: settings,
            text: ''
        });
    };
    IfUpdownInterfaces.prototype.removeInterface = function (iface) {
        var index = this._modifiedBlocks.findIndex(function (block) { return block.type === BlockType.iface && block.iface === iface; });
        if (index >= 0) {
            this._modifiedBlocks.splice(index, 1);
            return true;
        }
        return false;
    };
    return IfUpdownInterfaces;
}());
exports.IfUpdownInterfaces = IfUpdownInterfaces;
exports.default = IfUpdownInterfaces;
//# sourceMappingURL=index.js.map